package main

import (
	"fmt"
	"io/ioutil"
	"net/http"
	"net/http/httptest"
	"strings"
	"testing"

	"github.com/open-io/oio-exporter/common"
)

const HOSTNAME = "testnode"

func TestOIOExporter(t *testing.T) {
	t.Parallel()

	metrics := common.MakeMetricData()

	exp := &OioExporter{
		inventoryPath: "./tmp/inventory.yml",
		metrics:       metrics,
	}

	handler := oioExporterHandler{
		exporter:  exp,
		hostLabel: fmt.Sprintf(`host="%s"`, HOSTNAME),
	}

	server := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		switch r.URL.Path {
		case "/metrics":
			handler.handleExport(w, r)
		case "/config":
			handler.handleConfig(w, r)
		case "/reload":
			handler.handleReload(w, r)
		case "/daemons":
			handler.handleDaemons(w, r)
		}
	}))
	defer server.Close()

	url := server.URL

	for _, route := range []string{"/metrics", "/config", "/reload", "/daemons"} {
		resp, err := http.Get(url + route)
		if err != nil {
			log.Fatalln(err)
		}
		resp.Body.Close()
	}

	metrics.Set(&common.Metric{
		Name: "test_measurement",
		Labels: [][]string{
			[]string{"label1", "label1value"},
			[]string{"label2", "label2value"},
		},
		Value: float64(0.1),
	})

	resp, err := http.Get(url + "/metrics")
	if err != nil {
		log.Fatalln(err)
	}
	defer resp.Body.Close()
	data, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		log.Fatalln(err)
	}

	lines := strings.Split(string(data), "\n")

	// Check that all measures contain the hostname
	for _, line := range lines[:len(lines)-1] {
		if !strings.Contains(line, HOSTNAME) {
			log.Fatalln("Measurement", line, "must contain the host label", HOSTNAME)
		}
	}
}
